package egg_energy

import (
	"applet/app/db"
	"applet/app/e"
	md "applet/app/md/institutional_management/egg_energy"
	"applet/app/utils"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/enum"
	md2 "code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/md"
	"errors"
	"github.com/gin-gonic/gin"
	"time"
)

// GetVirtualCoinList
// @Summary      制度中心-蛋蛋能量-查询所有币种(获取)
// @Tags         蛋蛋能量
// @Description  查询所有币种(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Success      200   {object}  []md.VirtualCoin	"成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggEnergy/availableEnergy/list [post]
func GetVirtualCoinList(c *gin.Context) {
	virtualCoinDb := implement.NewVirtualCoinDb(db.Db)
	coins, err := virtualCoinDb.VirtualCoinFindAll()
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}

	list := make([]md.VirtualCoin, len(coins))
	for i, coin := range coins {
		list[i] = md.VirtualCoin{
			Id:            coin.Id,
			Name:          coin.Name,
			ExchangeRatio: coin.ExchangeRatio,
			IsUse:         coin.IsUse,
			CreateAt:      coin.CreateAt,
			UpdateAt:      coin.UpdateAt,
		}
	}

	e.OutSuc(c, list, nil)

}

// GetEggEnergyBasic
// @Summary      制度中心-蛋蛋能量-基础设置(获取)
// @Tags         蛋蛋能量
// @Description  基础设置(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Success      200   {object}   md.GetEggEnergyBasicResp  	"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggEnergy/getBasic [get]
func GetEggEnergyBasic(c *gin.Context) {
	basicSettingDb := implement.NewEggEnergyBasicSettingDb(db.Db)
	basicSettings, err1 := basicSettingDb.EggEnergyBasicSettingGetOne()
	if err1 != nil {
		e.OutErr(c, e.ERR_DB_ORM, err1.Error())
		return
	}
	if basicSettings == nil {
		e.OutErr(c, e.ERR_NO_DATA, errors.New("未能获取基础设置"))
		return
	}

	var videoRewardSystem md2.VideoRewardSystemStruct
	utils.Unserialize([]byte(basicSettings.VideoRewardSystem), videoRewardSystem)

	var destructionSetting md2.DestructionSettingStruct
	utils.Unserialize([]byte(basicSettings.DestructionSetting), destructionSetting)

	var priceSetting md2.PriceSettingStruct
	utils.Unserialize([]byte(basicSettings.PriceSetting), priceSetting)

	var exchangeRules md2.ExchangeRulesStruct
	utils.Unserialize([]byte(basicSettings.ExchangeRules), exchangeRules)

	var rewardSystem md2.RewardSystemStruct
	utils.Unserialize([]byte(basicSettings.RewardSystem), rewardSystem)

	var newUserRewardRules md2.NewUserRewardRules
	utils.Unserialize([]byte(basicSettings.NewUserIncentiveRules), newUserRewardRules)

	resp := md.GetEggEnergyBasicResp{
		BasicSetting: md.BasicSetting{
			IsOpen:                basicSettings.IsOpen,
			PersonEggEnergyCoinId: basicSettings.PersonEggEnergyCoinId,
			TeamEggEnergyCoinId:   basicSettings.TeamEggEnergyCoinId,
			PersonEggPointsCoinId: basicSettings.PersonEggPointsCoinId,
			TeamEggPointsCoinId:   basicSettings.TeamEggPointsCoinId,
			DirectPushReward:      basicSettings.DirectPushReward,
		},
		VideoRewardSetting: md.VideoRewardSetting{
			VideoRewardIsOpen: basicSettings.VideoRewardIsOpen,
			VideoRewardSystem: md2.VideoRewardSystemStruct{
				RewardValue:     videoRewardSystem.RewardValue,
				RewardTotalNum:  videoRewardSystem.RewardTotalNum,
				IntervalMinutes: videoRewardSystem.IntervalMinutes,
				EachRoundHour:   videoRewardSystem.EachRoundHour,
			},
		},
		DataSetting: md.DataSetting{
			TotalIssuanceAmount:        basicSettings.TotalIssuanceAmount,
			TotalTechnologyTeam:        basicSettings.TotalTechnologyTeam,
			TotalAngelInvestor:         basicSettings.TotalAngelInvestor,
			TotalOperateFund:           basicSettings.TotalOperateFund,
			TotalEcologicalDevelopment: basicSettings.TotalEcologicalDevelopment,
			TotalUserForPerson:         basicSettings.TotalUserForPerson,
			TotalUserForTeam:           basicSettings.TotalUserForTeam,
		},
		DestructionSetting: md2.DestructionSettingStruct{
			DestructionQuantity:     destructionSetting.DestructionQuantity,
			PublicWelfareAndCharity: destructionSetting.PublicWelfareAndCharity,
			DevelopmentCommittee:    destructionSetting.DevelopmentCommittee,
			StarLevelDividends:      destructionSetting.StarLevelDividends,
			MarketplaceMerchant:     destructionSetting.MarketplaceMerchant,
			CommunityDividends:      destructionSetting.CommunityDividends,
		},
		PriceSetting: md2.PriceSettingStruct{
			PriceHigherThanValue: priceSetting.PriceHigherThanValue,
			MarketplaceMerchantsNumsExchangeMarketplaceMerchantsFundValue: priceSetting.MarketplaceMerchantsNumsExchangeMarketplaceMerchantsFundValue,
			PriceBelowValue: priceSetting.PriceBelowValue,
			MarketplaceMerchantsFundExchangeMarketplaceMerchantsNumsValue: priceSetting.MarketplaceMerchantsFundExchangeMarketplaceMerchantsNumsValue,
		},
		ExchangeRules: md2.ExchangeRulesStruct{
			AutoExchangeNumsByPerson: exchangeRules.AutoExchangeNumsByPerson,
			AutoExchangeRateByPerson: exchangeRules.AutoExchangeRateByPerson,
			AutoExchangeNumsByTeam:   exchangeRules.AutoExchangeNumsByTeam,
			AutoExchangeRateByTeam:   exchangeRules.AutoExchangeRateByTeam,
		},
		RewardSystem: md2.RewardSystemStruct{
			Level:           rewardSystem.Level,
			RewardCondition: rewardSystem.RewardCondition,
			RewardValue:     rewardSystem.RewardValue,
		},
		NewUserIncentiveRules: md2.NewUserRewardRules{
			InviteUserRewardValue: newUserRewardRules.InviteUserRewardValue,
			RewardCoefficient:     newUserRewardRules.RewardCoefficient,
			ContinueDays:          newUserRewardRules.ContinueDays,
		},
		SystemID: basicSettings.Id,
	}

	e.OutSuc(c, resp, nil)
}

// UpdateEggEnergyBasic
// @Summary      制度中心-蛋蛋能量-基础设置(获取)
// @Tags         蛋蛋能量
// @Description  基础设置(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	md.UpdateEggEnergyBasicReq 	true  "system_id 必填"
// @Success      200   {int}   	"修改数据条数"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggEnergy/updateBasic [post]
func UpdateEggEnergyBasic(c *gin.Context) {
	var req *md.UpdateEggEnergyBasicReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}

	destructionSettingStr := utils.SerializeStr(req.DestructionSetting)
	priceSettingStr := utils.SerializeStr(req.PriceSetting)
	videoRewardSystemStr := utils.SerializeStr(req.VideoRewardSetting.VideoRewardSystem)
	exchangeRulesStr := utils.SerializeStr(req.ExchangeRules)
	rewardSystemStr := utils.SerializeStr(req.RewardSystem)
	newUserIncentiveRulesStr := utils.SerializeStr(req.NewUserIncentiveRules)

	m := model.EggEnergyBasicSetting{
		Id:                         req.SystemID,
		IsOpen:                     req.BasicSetting.IsOpen,
		PersonEggEnergyCoinId:      req.BasicSetting.PersonEggEnergyCoinId,
		TeamEggEnergyCoinId:        req.BasicSetting.TeamEggEnergyCoinId,
		PersonEggPointsCoinId:      req.BasicSetting.PersonEggPointsCoinId,
		TeamEggPointsCoinId:        req.BasicSetting.TeamEggPointsCoinId,
		TotalIssuanceAmount:        req.DataSetting.TotalIssuanceAmount,
		TotalTechnologyTeam:        req.DataSetting.TotalTechnologyTeam,
		TotalAngelInvestor:         req.DataSetting.TotalAngelInvestor,
		TotalOperateFund:           req.DataSetting.TotalOperateFund,
		TotalEcologicalDevelopment: req.DataSetting.TotalEcologicalDevelopment,
		TotalUserForPerson:         req.DataSetting.TotalUserForPerson,
		TotalUserForTeam:           req.DataSetting.TotalUserForTeam,
		DestructionSetting:         destructionSettingStr,
		DirectPushReward:           req.BasicSetting.DirectPushReward,
		PriceSetting:               priceSettingStr,
		VideoRewardIsOpen:          req.VideoRewardSetting.VideoRewardIsOpen,
		VideoRewardSystem:          videoRewardSystemStr,
		ExchangeRules:              exchangeRulesStr,
		RewardSystem:               rewardSystemStr,
		NewUserIncentiveRules:      newUserIncentiveRulesStr,
	}

	energyBasicSettingDb := implement.NewEggEnergyBasicSettingDb(db.Db)
	session := db.Db.NewSession()
	affected, err := energyBasicSettingDb.EggEnergyBasicSettingUpdate(session, req.SystemID, &m)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	e.OutSuc(c, affected, nil)
}

// GetEggCoreDataList
// @Summary      制度中心-蛋蛋能量-数据概览(获取)
// @Tags         蛋蛋能量
// @Description  数据概览(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	md.GetEggCoreDataListReq 	false  "起始时间请同时填写或同时不填"
// @Success      200   {object} md.GetEggCoreDataListResp   	"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggEnergy/globalData/coreDataList [post]
func GetEggCoreDataList(c *gin.Context) {
	var req *md.GetEggCoreDataListReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}

	coreDataDb := implement.NewEggEnergyCoreDataDb(db.Db)
	coreData, err := coreDataDb.EggEnergyCoreDataGet()
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	if coreData == nil {
		e.OutErr(c, e.ERR_NO_DATA, errors.New("数据不存在"))
		return
	}

	var startAt, endAt string
	if req.StartAt == "" || req.EndAt == "" {
		// 默认计算最近三十天 23 时的数据
		nowHour := time.Now().Hour()
		if nowHour >= 23 {
			req.EndAt = time.Now().Format("2006-01-02")
			req.StartAt = time.Now().Add(-30 * 24 * time.Duration(time.Hour)).Format("2006-01-02")
		} else {
			req.EndAt = time.Now().Add(-24 * time.Duration(time.Hour)).Format("2006-01-02")
			req.StartAt = time.Now().Add(-31 * 24 * time.Duration(time.Hour)).Format("2006-01-02")
		}
	} else {
		startAt = req.StartAt
		endAt = req.EndAt
	}

	hour := "23"
	energyPriceDb := implement.NewEggEnergyPriceDb(db.Db)
	priceList, err := energyPriceDb.EggEnergyPriceList(startAt, endAt, hour)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	prices := make([]md.PriceNode, len(priceList))
	for i, price := range priceList {
		prices[i].Date = price.Date
		prices[i].Price = price.Price
	}

	resp := md.GetEggCoreDataListResp{
		PlanetTotalValue:   coreData.PlanetTotalValue,
		NowPrice:           coreData.NowPrice,
		NowEnergyTotalNums: coreData.NowEnergyTotalNums,
		// todo 计算用户持有总量
		UserHoldTotalNums: "",
		Prices:            prices,
	}

	e.OutSuc(c, resp, nil)
}

// GetFundDataList
// @Summary      制度中心-蛋蛋能量-新增数据列表(获取)
// @Tags         蛋蛋能量
// @Description  新增数据列表(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	md.GetFundDataListReq 	true  "分页信息必填"
// @Success      200   {object} md.GetFundDataListResp   	"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggEnergy/globalData/fundDataList [post]
func GetFundDataList(c *gin.Context) {
	var req *md.GetFundDataListReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}

	var kindList = []map[string]interface{}{
		{
			"name":  enum.EggEnergyFundDataKind.String(enum.CompanySubsidies),
			"value": enum.CompanySubsidies,
		},
		{
			"name":  enum.EggEnergyFundDataKind.String(enum.InvestmentByInvestors),
			"value": enum.InvestmentByInvestors,
		},
		{
			"name":  enum.EggEnergyFundDataKind.String(enum.MarketExpectations),
			"value": enum.MarketExpectations,
		},
	}

	fundDataDb := implement.NewEggEnergyFundDataDb(db.Db)
	fundDataList, total, err := fundDataDb.EggEnergyFundDataFindAndCount(req.Page, req.Limit, req.Limit, req.StartAt, req.EndAt)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}

	list := make([]md.FundDataNode, len(fundDataList))
	for i, fundData := range fundDataList {
		list[i].Kind = fundData.Kind
		list[i].TotalAmount = fundData.TotalAmount
		list[i].Hours = fundData.Hours
		list[i].Memo = fundData.Memo
		list[i].BalanceTimes = fundData.BalanceTimes
		list[i].BalanceAmount = fundData.BalanceAmount
		list[i].CreateAt = fundData.CreateAt
		list[i].SystemID = fundData.Id
	}

	resp := md.GetFundDataListResp{
		KindList: kindList,
		Paginate: md.Paginate{
			Limit: req.Limit,
			Page:  req.Page,
			Total: total,
		},
		List: list,
	}
	e.OutSuc(c, resp, nil)
}

// GetFundDataRecordList
// @Summary      制度中心-蛋蛋能量-新增数据列表详情(查询)
// @Tags         蛋蛋能量
// @Description  新增数据列表详情(查询)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	md.GetFundDataRecordListReq 	true  "获取到的 system_id 以及分页信息"
// @Success      200   {object} md.GetFundDataRecordListResp   	"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggEnergy/globalData/fundDataList [post]
func GetFundDataRecordList(c *gin.Context) {
	var req *md.GetFundDataRecordListReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}

	recordsDb := implement.NewEggEnergyFundDataRecordsDb(db.Db)
	records, total, err := recordsDb.EggEnergyFundDataRecordsFindAndCount(req.Page, req.Limit, req.SystemID)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	var list []md.FundDataRecordNode
	for i, record := range records {
		list[i].TotalAmount = record.TotalAmount
		list[i].BalanceAmount = record.BalanceAmount
		list[i].BalanceTimes = record.BalanceTimes
		list[i].BeforePrice = record.BeforePrice
		list[i].AfterPrice = record.AfterPrice
		list[i].BeforePlanetTotalValue = record.BeforePlanetTotalValue
		list[i].AfterPlanetTotalValue = record.AfterPlanetTotalValue
		list[i].CreateAt = record.CreateAt
	}

	resp := md.GetFundDataRecordListResp{
		Paginate: md.Paginate{
			Limit: req.Limit,
			Page:  req.Page,
			Total: total,
		},
		List: list,
	}

	e.OutSuc(c, resp, nil)
}
